@@ -3,12 +3,23 @@ from flask_discord_interactions.models.user import User, Member
 from flask_discord_interactions.models.message import Message, ResponseType
 from flask_discord_interactions.models.embed import Embed
 from flask_discord_interactions.models.component import (
-    ComponentType, ActionRow, Button, SelectMenu, SelectMenuOption, Component, ButtonStyles
+    ComponentType,
+    ActionRow,
+    Button,
+    SelectMenu,
+    SelectMenuOption,
+    Component,
+    ButtonStyles,
 )
-from flask_discord_interactions.models.command import ApplicationCommandType, CommandOptionType
+from flask_discord_interactions.models.command import ApplicationCommandType
 from flask_discord_interactions.models.permission import Permission
 from flask_discord_interactions.models.role import Role
 from flask_discord_interactions.models.utils import LoadableDataclass
+from flask_discord_interactions.models.autocomplete import (
+    Autocomplete,
+    AutocompleteResult,
+)
+from flask_discord_interactions.models.option import Option, CommandOptionType
 
 __all__ = [
     "Channel",
@@ -30,4 +41,7 @@ __all__ = [
     "Permission",
     "Role",
     "LoadableDataclass",
-]
\ No newline at end of file
+    "Autocomplete",
+    "AutocompleteResult",
+    "Option",
+]
